extern crate bcrypt;

use crate::models;
use bcrypt::{hash, DEFAULT_COST};
use diesel::prelude::*;

pub fn insert_new_user(
    name: &str,
    password: &str,
    conn: &MysqlConnection,
) -> Result<models::NewUser, diesel::result::Error> {
    use crate::schema::users::dsl::*;

    let new_user = models::NewUser {
        username: name.to_owned(),
        password_hash: hash(password, DEFAULT_COST).unwrap(),
        is_admin: false,
    };

    diesel::insert_into(users).values(&new_user).execute(conn)?;
    Ok(new_user)
}

pub fn find_user_by_name(
    name: &str,
    conn: &MysqlConnection,
) -> Result<Option<models::User>, diesel::result::Error> {
    use crate::schema::users::dsl::*;

    let user = users
        .filter(username.eq(name))
        .first::<models::User>(conn)
        .optional()?;

    Ok(user)
}

pub fn find_user_by_id(
    id: u16,
    conn: &MysqlConnection,
) -> Result<Option<models::User>, diesel::result::Error> {
    use crate::schema::users::dsl::*;

    let user = users
        .filter(id.eq(id))
        .first::<models::User>(conn)
        .optional()?;

    Ok(user)
}
