#[macro_use]
extern crate diesel;

use actix_web::{middleware, post, web, App, Error, HttpResponse, HttpServer};
use diesel::prelude::*;
use diesel::r2d2::{self, ConnectionManager};

mod actions;
mod models;
mod schema;

type DbPool = r2d2::Pool<ConnectionManager<MysqlConnection>>;

#[post("/users/add")]
async fn add_user(
    pool: web::Data<DbPool>,
    form: web::Json<models::NewUser>,
) -> Result<HttpResponse, Error> {
    let conn = pool.get().expect("Pool did not give a connection");
    println!("Hi");

    // use web::block to offload blocking Diesel code without blocking server thread
    let user =
        web::block(move || actions::insert_new_user(&form.username, &form.password_hash, &conn))
            .await
            .map_err(|e| {
                eprint!("{}", e);
                HttpResponse::InternalServerError().finish()
            })?;

    Ok(HttpResponse::Ok().json(user))
}

#[actix_rt::main]
async fn main() -> std::io::Result<()> {
    std::env::set_var("RUST_LOG", "actix_web=info,diesel=debug");
    dotenv::dotenv().ok();
    env_logger::init();

    let connspec = std::env::var("DATABASE_URL").expect("DATABASE_URL");
    let manager = ConnectionManager::<MysqlConnection>::new(connspec);
    let pool = r2d2::Pool::builder()
        .build(manager)
        .expect("Failed to create database connection pool");

    let bind = "127.0.0.1:8080";
    println!("Starting server at: http://{}", &bind);

    HttpServer::new(move || {
        App::new()
            .data(pool.clone())
            .wrap(middleware::Logger::default())
            .service(add_user(pool, form))
    })
    .bind(&bind)?
    .run()
    .await
}
